from typing import Dict

from nonebot import get_bot, get_driver, on_regex
from nonebot.exception import FinishedException
from nonebot.log import logger
from nonebot.typing import T_State
from nonebot_plugin_apscheduler import scheduler

try:
    from nonebot.adapters.onebot.v11 import Bot, Event, Message  # type: ignore
    from nonebot.adapters.onebot.v11.event import (  # type: ignore
        GroupMessageEvent,
        MessageEvent,
    )
except ImportError:
    from nonebot.adapters.cqhttp import Bot, Event, Message
    from nonebot.adapters.cqhttp.event import GroupMessageEvent, MessageEvent

from .data_source import getEpicFree, subscribeHelper

try:
    epicScheduler = get_driver().config.epic_scheduler
    assert epicScheduler is not None
except (AttributeError, AssertionError):
    epicScheduler = "5 8 8 8"
day_of_week, hour, minute, second = epicScheduler.split(" ")


epicMatcher = on_regex("((E|e)(P|p)(I|i)(C|c))?喜(加一|\+1)", priority=2)


@epicMatcher.handle()
async def onceHandle(bot: Bot, event: Event):
    imfree = await getEpicFree()
    await epicMatcher.finish(Message(imfree))


epicSubMatcher = on_regex("喜(加一|\+1)(私聊)?订阅", priority=1)


@epicSubMatcher.handle()
async def subHandle(bot: Bot, event: MessageEvent, state: T_State):
    if isinstance(event, GroupMessageEvent):
        if event.sender.role not in ["admin", "owner"] or "私聊" in event.get_plaintext():
            # 普通群员只会启用私聊订阅
            # state["targetId"] = event.get_user_id()
            state["subType"] = "私聊"
        else:
            # 管理员用户询问需要私聊订阅还是群聊订阅
            pass
    else:
        state["subType"] = "私聊"


@epicSubMatcher.got("subType", prompt="默认启用群聊订阅，如需私聊订阅请回复「私聊」")
async def subEpic(bot: Bot, event: MessageEvent, state: T_State):
    if "私聊" in state["subType"]:
        state["targetId"] = event.get_user_id()
        state["subType"] = "私聊"
    else:
        state["targetId"] = str(event.group_id)  # type: ignore
        state["subType"] = "群聊"
    msg = await subscribeHelper("w", state["subType"], state["targetId"])
    assert isinstance(msg, str)
    await epicSubMatcher.finish(msg)


@scheduler.scheduled_job(
    "cron", day_of_week=day_of_week, hour=hour, minute=minute, second=second
)
async def weeklyEpic():
    bot = get_bot()
    whoSubscribe = await subscribeHelper()
    imfree = await getEpicFree()
    try:
        assert isinstance(whoSubscribe, Dict)
        for group in whoSubscribe["群聊"]:
            await bot.send_group_msg(group_id=group, message=Message(imfree))
        for private in whoSubscribe["私聊"]:
            await bot.send_private_msg(user_id=private, message=Message(imfree))
    except FinishedException:
        pass
    except Exception as e:
        logger.error(f"Epic 限免游戏资讯定时任务出错 {type(e)}：{e}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_epicfree']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.0.0',
 'nonebot-adapter-onebot>=2.0.0b1',
 'nonebot-plugin-apscheduler>=0.1.0',
 'nonebot2>=2.0.0a14']

setup_kwargs = {
    'name': 'nonebot-plugin-epicfree',
    'version': '0.1.8',
    'description': 'A Epic free game info plugin for Nonebot2',
    'long_description': '<h1 align="center">Nonebot Plugin EpicFree</h1></br>\n\n\n<p align="center">🤖 用于获取 Epic 限免游戏资讯的 Nonebot2 插件</p></br>\n\n\n<p align="center">\n  <a href="https://github.com/monsterxcn/nonebot_plugin_epicfree/actions">\n    <img src="https://img.shields.io/github/workflow/status/monsterxcn/Typecho-Theme-VOID/Build?style=flat-square" alt="actions">\n  </a>\n  <a href="https://raw.githubusercontent.com/monsterxcn/nonebot_plugin_epicfree/master/LICENSE">\n    <img src="https://img.shields.io/github/license/monsterxcn/nonebot_plugin_epicfree?style=flat-square" alt="license">\n  </a>\n  <a href="https://pypi.python.org/pypi/nonebot_plugin_epicfree">\n    <img src="https://img.shields.io/pypi/v/nonebot_plugin_epicfree?style=flat-square" alt="pypi">\n  </a>\n  <img src="https://img.shields.io/badge/python-3.7.3+-blue?style=flat-square" alt="python"><br />\n</p></br>\n\n\n**安装方法**\n\n\n使用以下命令之一快速安装（若配置了 PyPI 镜像，你可能无法及时检索到插件最新版本）：\n\n\n``` zsh\nnb plugin install nonebot_plugin_epicfree\n\npip install --upgrade nonebot_plugin_epicfree\n```\n\n\n重启 Bot 即可体验此插件。\n\n\n<details><summary><i>关于 nonebot2 及相关依赖版本</i></summary></br>\n\n\n此插件在以下两种环境测试通过！\n\n\n```\n# 文件夹拷贝方式安装\nnonebot2.0.0.a16 nonebot-adapter-cqhttp==2.0.0a16\n\n# 命令行 / 脚手架安装\nnonebot2.0.0.b2 nonebot-adapter-onebot==2.0.0b1\n```\n\n\n此插件在 nonebot2.0.0.b1 下可能不兼容，需要参考 [commit `44f4bf8`](https://github.com/monsterxcn/nonebot_plugin_epicfree/commit/44f4bf8c3c578fff242a106a28b85884c78a0404) 自行修改 `__init__.py` 中 `T_State` 的写法。\n\n在已淘汰的 Nonebot2 适配器 [nonebot-adapter-cqhttp](https://pypi.org/project/nonebot-adapter-cqhttp/) 下，通过拷贝文件夹 `nonebot_plugin_epicfree` 至 Nonebot2 插件目录、手动安装 `nonebot-plugin-apscheduler` 和 `httpx` 依赖后仍可正常启用此插件。在未来某个版本会完全移除该适配器支持，请尽快升级至 `nonebot-adapter-onebot`。\n\n以上述命令行方式安装本插件时，可能由于依赖版本差异报错。对于新手，推荐在安装插件前备份当前环境依赖版本，以便后续恢复：\n\n\n```bash\n# 备份当前的依赖版本\npip3 freeze > requirements.txt\n\n# 尝试安装 nonebot_plugin_epicfree\n\n# 若安装出错，可尝试恢复之前备份的依赖版本\npip3 install -r requirements.txt\n```\n\n\n</details>\n\n\n<details><summary><i>单独加载此插件</i></summary></br>\n\n\n在 Nonebot2 入口文件（例如 `bot.py`）增加：\n\n\n``` python\nnonebot.load_plugin("nonebot_plugin_epicfree")\n```\n\n\n</details>\n\n\n**使用方法**\n\n\n```python\n# nonebot_plugin_epicfree/__init__.py#L27\nepicMatcher = on_regex("((E|e)(P|p)(I|i)(C|c))?喜(加一|\\+1)")\n\n# nonebot_plugin_epicfree/__init__.py#L34\nepicSubMatcher = on_regex("喜(加一|\\+1)(私聊)?订阅")\n```\n\n\n发送「喜加一」查找游戏，群组内发送「喜加一订阅」订阅限免游戏资讯。基于正则匹配，所以，甚至「EpIc喜+1」这样的指令都可用！（\n\n限免游戏资讯订阅功能默认在插件文件夹内生成配置文件。但建议自行指定用于存放订阅配置的文件夹，将其写入 `resources_dir` 环境变量即可。注意该文件夹需要包含一个名为 `epicfree` 的子文件夹。在 Nonebot2 `.env` 中填写时注意去除结尾的 `/`，如果是 Windows 系统应写成形如 `D:/path/to/resources_dir`。\n\n限免游戏资讯订阅默认每周六 08:08:08 发送，如需自定义请在 `.env` 中添加格式如下的配置，其中四个数字依次代表 `day_of_week` `hour` `minute` `second`。\n\n\n```\nresources_dir="/data/bot/resources"\nepic_scheduler="5 8 8 8"\n```\n\n\n**特别鸣谢**\n\n\n[@nonebot/nonebot2](https://github.com/nonebot/nonebot2/) | [@Mrs4s/go-cqhttp](https://github.com/Mrs4s/go-cqhttp) | [@DIYgod/RSSHub](https://github.com/DIYgod/RSSHub) | [@SD4RK/epicstore_api](https://github.com/SD4RK/epicstore_api)\n\n\n> 作者是 Nonebot 新手，代码写的较为粗糙，欢迎提出修改意见或加入此插件开发！溜了溜了...\n',
    'author': 'monsterxcn',
    'author_email': 'monsterxcn@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/monsterxcn/nonebot_plugin_epicfree',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<4.0',
}


setup(**setup_kwargs)



import json
import os
from datetime import datetime
from traceback import format_exc
from typing import Dict, List, Union

from httpx import AsyncClient
from nonebot import get_driver
from nonebot.log import logger

try:
    resPath = get_driver().config.resources_dir
    cache = f"{resPath}{os.sep}epicfree{os.sep}status.json"
    assert os.path.exists(cache)
except (AttributeError, AssertionError):
    resPath = os.path.dirname(os.path.abspath(__file__))
    cache = f"{resPath}{os.sep}status.json"


# 写入与读取订阅信息
# method="w" 写入时返回新增订阅结果字符串
# method="r" 读取时返回订阅状态字典
async def subscribeHelper(
    method: str = "r", subType: str = "", subject: str = ""
) -> Union[Dict, str]:
    statusDict = {"群聊": [], "私聊": []}
    try:
        with open(cache, "r", encoding="UTF-8") as f:
            statusDict = json.load(f)
    except FileNotFoundError:
        with open(cache, "w", encoding="UTF-8") as f:
            json.dump(statusDict, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error(f"获取 Epic 订阅 JSON 错误 {type(e)}：{e}")
    # 读取时，返回订阅状态字典
    if method != "w":
        return statusDict
    # 写入时，将新的用户按类别写入至指定数组
    try:
        if subject in statusDict[subType]:
            return f"{subType}已经订阅过 Epic 限免游戏资讯了哦！"
        statusDict[subType].append(subject)
        with open(cache, "w", encoding="UTF-8") as f:
            json.dump(statusDict, f, ensure_ascii=False, indent=2)
        return f"{subType}订阅 Epic 限免游戏资讯成功！"
    except Exception as e:
        logger.error(f"获取 Epic 订阅 JSON 错误 {type(e)}：{e}")
        return f"{subType}订阅 Epic 限免游戏资讯失败惹.."


# 获取所有 Epic Game Store 促销游戏
# 方法参考：RSSHub /epicgames 路由
# https://github.com/DIYgod/RSSHub/blob/master/lib/routes/epicgames/index.js
async def getEpicGame() -> List:
    epic_url = (
        "https://store-site-backend-static-ipv4.ak.epicgames.com"
        "/freeGamesPromotions?locale=zh-CN&country=CN&allowCountries=CN"
    )
    headers = {
        "Referer": "https://www.epicgames.com/store/zh-CN/",
        "Content-Type": "application/json; charset=utf-8",
        "User-Agent": (
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36"
            " (KHTML, like Gecko) Chrome/98.0.4758.80 Safari/537.36"
        ),
    }
    async with AsyncClient(proxies={"all://": None}) as client:
        try:
            res = await client.get(epic_url, headers=headers, timeout=10.0)
            resJson = res.json()
            games = resJson["data"]["Catalog"]["searchStore"]["elements"]
            # with open(f"{resPath}epicfree/api.json", "w", encoding="utf-8") as f:
            #     json.dump(games, f, ensure_ascii=False, indent=2)
            return games
        except Exception as e:
            logger.error(f"请求 Epic Store API 错误 {type(e)}：{e}")
            return []


# 获取 Epic Game Store 免费游戏信息
# 处理免费游戏的信息方法借鉴 pip 包 epicstore_api 示例
# https://github.com/SD4RK/epicstore_api/blob/master/examples/free_games_example.py
async def getEpicFree() -> str:
    games = await getEpicGame()
    if not games:
        return "Epic 可能又抽风啦，请稍后再试（"
    else:
        logger.debug(f"获取到游戏：{('、'.join(game['title'] for game in games))}")
        msgList = []
        for game in games:
            game_name = ""
            try:
                game_name = game["title"]
                game_corp = game["seller"]["name"]
                game_price = game["price"]["totalPrice"]["fmtPrice"]["originalPrice"]
                game_promotions = game["promotions"]["promotionalOffers"]
                upcoming_promotions = game["promotions"]["upcomingPromotionalOffers"]
                game_thumbnail, game_dev, game_pub = None, game_corp, game_corp
                if not game_promotions and upcoming_promotions:
                    logger.info(f"即将推出免费游玩的游戏：{game_name}({game_price})")
                    continue  # 促销即将上线，跳过
                else:
                    for image in game["keyImages"]:
                        # 修复部分游戏无法找到图片
                        # https://github.com/HibiKier/zhenxun_bot/commit/92e60ba141313f5b28f89afdfe813b29f13468c1
                        if (
                            image.get("url")
                            and not game_thumbnail
                            and image["type"]
                            in [
                                "Thumbnail",
                                "VaultOpened",
                                "DieselStoreFrontWide",
                                "OfferImageWide",
                            ]
                        ):
                            game_thumbnail = image["url"]
                            break
                    for pair in game["customAttributes"]:
                        if pair["key"] == "developerName":
                            game_dev = pair["value"]
                        elif pair["key"] == "publisherName":
                            game_pub = pair["value"]
                    game_desp = game["description"]
                    date_iso = game_promotions[0]["promotionalOffers"][0]["endDate"][:-1]
                    end_date = datetime.fromisoformat(date_iso).strftime("%b.%d %H:%M")
                    # API 返回不包含游戏商店 URL，此处自行拼接，可能出现少数游戏 404 请反馈
                    if game.get("productSlug"):
                        game_url = "https://store.epicgames.com/zh-CN/p/{}".format(
                            game["productSlug"].replace("/home", "")
                        )
                    elif game.get("url"):
                        game_url = game["url"]
                    else:
                        slugs = (
                            [
                                x["pageSlug"]
                                for x in game.get("offerMappings", [])
                                if x.get("pageType") == "productHome"
                            ]
                            + [
                                x["pageSlug"]
                                for x in game.get("catalogNs", {}).get("mappings", [])
                                if x.get("pageType") == "productHome"
                            ]
                            + [
                                x["value"]
                                for x in game.get("customAttributes", [])
                                if "productSlug" in x.get("key")
                            ]
                        )
                        game_url = "https://store.epicgames.com/zh-CN{}".format(
                            f"/p/{slugs[0]}" if len(slugs) else ""
                        )
                    msgList.append(
                        (
                            "{}FREE now :: {} ({})\n\n{}\n\n{}，"
                            "将在 UTC 时间 {} 结束免费游玩，戳链接领取吧~\n{}"
                        ).format(
                            (
                                f"[CQ:image,file={game_thumbnail}]\n\n"
                                if game_thumbnail
                                else ""
                            ),
                            game_name,
                            game_price,
                            game_desp,
                            (
                                f"游戏由 {game_pub} 发行"
                                if game_dev == game_pub
                                else f"游戏由 {game_dev} 开发、{game_pub} 发行"
                            ),
                            end_date,
                            game_url,
                        )
                    )
            except (AttributeError, IndexError, TypeError):
                logger.debug(f"处理游戏 {game_name} 时遇到应该忽略的错误\n{format_exc()}")
                pass
            except Exception as e:
                logger.error(f"组织 Epic 订阅消息错误 {type(e)}\n{format_exc()}")
        # 返回整理为 CQ 码的消息字符串
        msg = "\n\n---\n\n".join(msgList) if len(msgList) else "暂未找到正在促销的游戏..."
        return msg
